//+------------------------------------------------------------------+
//|                                  Elders's market thermometer.mq4 |
//|                                                  coded by mladen |
//|                                                                  |
//|                                                                  |
//| Original developed by Alexander Elder,                           |
//| "Come Into My Trading Room"                                      |
//|                                                                  |
//|                                                                  |
//| modes :                                                          |
//|   1 - original mode                                              |
//|   2 - change in colors:                                          |
//|       green -> up change                                         |
//|       red -> down change                                         |
//|   3 - Silberman's and Lynch modifications                        |
//|       if in trade already and red bar, then exit                 |
//|       get ready for explosive move if yellow bar                 |
//|       if not in trade, white bar suggests a move has started     |
//|       set profit target for next day                             |
//|         -- if Long, add value of today's Thermometer EMA to      |
//|            yesterday's high and place sell order there           |
//|         -- if Short, subtract value of today's Thermometer EMA   |
//|            from yesterday's low and place order to cover there   |
//|                                                                  |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 8
#property indicator_minimum 0
#property indicator_color1  Gold
#property indicator_color2  Gold
#property indicator_color3  Gold
#property indicator_color4  Green
#property indicator_color5  Red
#property indicator_color6  LimeGreen
#property indicator_color7  Orange
#property indicator_color8  White
#property indicator_style2  STYLE_DOT
#property indicator_style3  STYLE_DOT
#property indicator_width4  2
#property indicator_width5  2
#property indicator_width6  2
#property indicator_width7  2
#property indicator_width8  2

//
//
//
//
//

extern int EMALength = 22;
extern int EMAKoef1  = 2;
extern int EMAKoef2  = 3;
extern int BarsUnder = 5;
extern int Mode      = 3;

//
//
//
//
//

double avgTemperatureA[];
double avgTemperatureB[];
double avgTemperatureC[];
double temperature[];
double buffer5[];
double buffer6[];
double buffer7[];
double buffer8[];


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,avgTemperatureA);
   SetIndexBuffer(1,avgTemperatureB);
   SetIndexBuffer(2,avgTemperatureC);
   SetIndexBuffer(3,temperature);
   SetIndexBuffer(4,buffer5);
   SetIndexBuffer(5,buffer6);
   SetIndexBuffer(6,buffer7);
   SetIndexBuffer(7,buffer8);

   for(int i=3; i<8; i++)
             SetIndexStyle(i,DRAW_HISTOGRAM);
             SetIndexEmptyValue(0,0.00);
             SetIndexEmptyValue(3,0.00);
   Mode = MathMax(MathMin(3,Mode),1);
   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   double alpha     = 2.0/(1.0+EMALength);
   int counted_bars = IndicatorCounted();
   int limit,i;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = Bars-counted_bars;
         limit = MathMin(limit,Bars-2);

   //
   //
   //
   //
   //
   
   for(i = limit; i >= 0; i--)
   {
      temperature[i] = 0.00;
         if ((High[i]-High[i+1])  > (Low[i+1]-Low[i])) temperature[i] = MathAbs(High[i]-High[i+1]);
         if ((High[i]-High[i+1])  < (Low[i+1]-Low[i])) temperature[i] = MathAbs(Low[i+1]-Low[i]);
         if ((High[i]<High[i+1]) && (Low[i]>Low[i+1])) temperature[i] = 0.00;
      avgTemperatureA[i] = avgTemperatureA[i+1]+alpha*(temperature[i]-avgTemperatureA[i+1]);
      
      //
      //
      //
      //
      //
         
      if (Mode!=3)
         {
            avgTemperatureB[i] = avgTemperatureA[i]*EMAKoef1;
            avgTemperatureC[i] = avgTemperatureA[i]*EMAKoef2;
            buffer5[i]         = EMPTY_VALUE;
      
            if (Mode==2)
               if((High[i]-High[i+1]) < (Low[i+1]-Low[i]))
                  buffer5[i] = temperature[i];
         }
      else
         {
            avgTemperatureB[i] = avgTemperatureA[i]*EMAKoef2;
      
            //
            //
            //
            //
            //
      
            int whichBuffer = 1;
            
            if (temperature[i] < avgTemperatureA[i]) whichBuffer = 1;
               else if(temperature[i] < avgTemperatureB[i]) whichBuffer = 2;
                  else if(temperature[i] > avgTemperatureB[i]) whichBuffer = 3;
   
            //
            //
            //
            // 
            //

            int scoreC = 0;
            int scoreP = 0;
               for (int k=0;k<BarsUnder;k++)
               {
                  if(temperature[i+k]   < avgTemperatureA[i+k])   scoreC += 1;
                  if(temperature[i+k+1] < avgTemperatureA[i+k+1]) scoreP += 1;
               }            
               if(scoreC >= BarsUnder) whichBuffer = 4;
               if(scoreP >= BarsUnder && temperature[i] >= avgTemperatureA[i] && temperature[i] < avgTemperatureB[i]) whichBuffer = 5;

               buffer5[i] = EMPTY_VALUE;
               buffer6[i] = EMPTY_VALUE;
               buffer7[i] = EMPTY_VALUE;
               buffer8[i] = EMPTY_VALUE;
               switch (whichBuffer)
               {
                  case 2: { buffer6[i]=temperature[i]; break; }
                  case 3: { buffer5[i]=temperature[i]; break; }
                  case 4: { buffer7[i]=temperature[i]; break; }
                  case 5: { buffer8[i]=temperature[i]; break; }
               }
         }
   }
   return(0);
}

/*
Use it like this from the book:-

MARKET THERMOMETER

A beginner needs to learn a battery of standard technical indicators and
start tweaking their parameters. Some traders create their own indicators
to gauge different aspects of crowd behavior and identify market
moves. Let us walk through the process of creating a new technical indicator
and see how to go about building your own private indicators.
All good indicators reflect some aspect of market reality. Market
Thermometer helps differentiate between sleepy, quiet periods and hot
episodes when market crowds become excited. It can help you adapt
your trading to the current environment.
Quiet markets typically have narrow bars that tend to overlap one
another. Hot, boiling markets tend to have wide bars whose highs and
lows extend far outside the previous days range. Beginners jump into
trades during those wide bars, afraid to miss a runaway move. If you
enter when the markets are quiet, your slippage is likely to be lower.
Hot markets are good for taking profits because then slippage may
work in your favor.
When gold recently ran up $40 in one week, a journalist asked a
famous investor whether it was a good buy. Gold was good, he said,
but the time to get on this bus was when it stood in front of the station,
and not rolling down the highway at 40 miles an hour. Market
Thermometer helps you recognize when the bus slows down in front
of the station, picks up speed, or roars down the highway.
Market Thermometer measures how far the most extreme point of
today, either high or low, protrudes outside of yesterdays range. The
greater the extension of todays bar outside of yesterdays, the higher
the market temperature. Heres the formula of Market Thermometer:
Temperature = the greater of either (Hightoday - Highyesterday)
or (Lowyesterday - Lowtoday)
To program Market Thermometer into Windows on WallStreet software,
use the following formula:
if (hi<ref(hi,-1) and lo>ref(lo,-1), 0, if ((hi - ref(hi,-1)) >
(ref(lo,-1) - lo), hi - ref(hi,-1), ref(lo,-1) - lo)).
It is easy to adapt this formula to other software packages.
Market temperature is always a positive number, reflecting the absolute
value of either the upward or the downward extension of yesterdays
range, whichever is greater. Plot temperature as a histogram
above zero. Calculate a moving average of market temperature, and
plot it as a line on the same chart. I use a 22-day EMA because there
are 22 trading days in a month, but feel free to experiment with shorter
EMA values if you want to make this indicator more sensitive to shortterm
swings.
When markets are quiet, the adjacent bars tend to overlap. The consensus
of value is well established, and the crowd does little buying
or selling outside of yesterdays range. When highs and lows exceed
their previous days values, they do so only by small margins. Market
Thermometer falls and its EMA slants down, indicating a sleepy market.
When a market begins to run, either up or down, its daily bars start
pushing outside of the previous ranges. The histogram of Market Thermometer
grows taller and crosses above its EMA, which soon turns up,
confirming the new trend.
Market Thermometer gives four trading signals, based on the relationship
between its histogram and its moving average:
The best time to enter new positions is when Market Thermometer
falls below its moving average. When Market Thermometer falls below
its EMA, it indicates that the market is quiet. If your system flashes an
entry signal, try to enter when the market is cooler than usual. When
Market Thermometer rises above its moving average, it warns that the
market is hot and slippage more likely.
Exit positions when Market Thermometer rises to triple the height
of its moving average. A spike of Market Thermometer indicates a
runaway move. When the crowd feels jarred by a sudden piece of
news and surges, it is a good time to take profits. Panics tend to be
short-lived, offering a brief opportunity to cash in. If the EMA of
Market Thermometer stands at 5 cents, but the Thermometer itself
shoots up to 15 cents, take profits. Test these values for the market
you are trading.

Get ready for an explosive move if the Thermometer stays below its
moving average for five to seven trading days. Quiet markets put amateurs
to sleep. They become careless and stop watching prices. Volatility
and volume fall, and professionals get a chance to run away with the
market. Explosive moves often erupt from periods of inactivity.
Market Thermometer can help you set a profit target for the next
trading day. If you are a short-term trader and are long, add the value
of todays Thermometer EMA to yesterdays high and place a sell order
there. If you are short, subtract the value of the Thermometers EMA
from yesterdays low and place an order to cover at that level.
I had two purposes in presenting the Market Thermometer. I wanted
to give you a new indicator, but even more I wanted to show you how
to use your understanding of the markets to design your own analytic
tools. Once you understand the principles of market analysis, you can
create your own indicators. Use your knowledge, understanding, and
discipline to get on the right side of the markets.

*/

